package info.kofler

import io.ktor.application.*
import io.ktor.features.origin
import io.ktor.html.*
import io.ktor.http.*
import io.ktor.response.*
import io.ktor.request.*
import io.ktor.routing.*
import io.ktor.util.pipeline.*
import kotlinx.html.*
import java.net.URL

// statt einer Datenbank
val urlMap = mutableMapOf<String, String>()

// HTML-Code für Formular
val shortenForm = """
    <html><body><form action='/shorten' method='post'>
    <p><label for='url'>Geben Sie hier Ihre URL an:</label>
    <p><input type='text' name='url' id='url' size='100'>
    <p><input type='submit' value='OK'>
    </form></body></html>
""".trimIndent()

fun main(args: Array<String>): Unit = io.ktor.server.netty.EngineMain.main(args)

@Suppress("unused") // Referenced in application.conf
@kotlin.jvm.JvmOverloads
fun Application.module(testing: Boolean = false) {
    routing {
        get("/") {
            // Formular anzeigen
            call.respondText(shortenForm, ContentType.Text.Html)
        }
        get("/{id}") {
            // URL suchen
            lookupUrl()
        }
        post("/shorten") {
            // Formulardaten verarbeiten, Short-URL erzeugen
            shortenUrl()
        }
    }
}

// Formularauswertung
suspend inline fun PipelineContext<Unit, ApplicationCall>.shortenUrl() {
    val req = call.request
    val host = "%s://%s:%s".format(req.origin.scheme, req.host(), req.port())
    val data = call.receiveParameters()
    val url = data["url"]
    if (url is String) {
        // simple Validierung
        val result = runCatching { URL(url) }
        if (result.isSuccess) {
            val rid: String
            // existiert die URL schon in der Map?
            val keys = urlMap.filterValues { it == url }.keys
            if (keys.isNotEmpty())
                // ja, Key weiterverwenden
                rid = keys.first()
            else {
                // nein, neuen Key erzeugen
                rid = randomId()
                urlMap[rid] = url
            }
            val href = "$host/$rid"
            call.respondHtml {
                body {
                    p { +"Original URL: "
                        a(url) { +url }
                    }
                    p { +"Shortened URL: "
                        a(href) { +href }
                    }
                    p { +"Shortened URL as text: $href" }
                }
            }
            return
        }
    }
    call.respondText("Invalid data.")
}


suspend inline fun PipelineContext<Unit, ApplicationCall>.lookupUrl() {
    val id = call.parameters["id"]
    if (id is String && urlMap.containsKey(id)) {
        val href = urlMap[id]!!
        // falls direkte Weiterleitung erwünscht ist
        // context.call.respondRedirect(href)
        call.respondHtml {
            body {
                p { +"Go to: "
                    a(href) { +href }
                }
            }
        }
    } else {
        call.respondText { "Invalid link." }
    }

}

// ca. 1,8 Mrd. Möglichkeiten; die Funktion wird
// zunehmend langsam, wenn ca. 50% der Möglichkeiten
// erschöpft sind
fun randomId() : String {
    val chars = "abcdefghijklmnopqrstuvwxyz0123456789"
    var id: String
    do {
        id = (1..6).map { chars.random() }.joinToString("")
    } while (urlMap.containsKey(id))
    urlMap[id] = "reserved"
    return id
}