package info.kofler.eval

import io.ktor.application.*
import io.ktor.auth.*
import io.ktor.features.*
import io.ktor.routing.*
import io.ktor.serialization.*


// Datenbank erzeugen/einrichten
val db = Db()   // Datei db.kt

fun main(args: Array<String>): Unit = io.ktor.server.netty.EngineMain.main(args)

@Suppress("unused") // Referenced in application.conf
@kotlin.jvm.JvmOverloads
fun Application.module(testing: Boolean = false) {
    install(ContentNegotiation) { json() }
    install(Authentication) {
        // Authentifizierungsmechanismus für App
        // liefert UserIdPricincipal mit Students.id (als Zeichenkette)
        // des authentifizierten Studenten zurück
        basic("appauth") {
            realm = "Evaluation Server / API"
            validate { cred ->
                val id = db.standardLogin(cred.name, cred.password)
                if (id is Int)
                    UserIdPrincipal(id.toString())
                else
                    null
            }
        }
    }

    routing {
        webrouting()  // Datei web.kt
        apirouting()  // Datei api.kt
    }
}

