package info.kofler.eval

import io.ktor.application.*
import io.ktor.auth.*
import io.ktor.request.*
import io.ktor.response.*
import io.ktor.routing.*
import kotlinx.serialization.*

@Serializable
data class LoginData (val studentNo: String, val onetimekey: String, val appkey: String)

fun Routing.apirouting() {
    route("/api") {
        version()   // Verbindungstest
        login()     // Einmal-Login

        // die folgenden Requests nur mit Authentifizierung
        // Code für "appauth"-Authentifizierung siehe Application.kt
        authenticate("appauth") {
            authtest()     // Verbindungstest
            evaluations()  // Liste der Evaluierungen
            questions()    // Liste der Fragen
            evaluation()   // Evaluierung speichern
            results()      // Evaluierungsergebnisse
        }
    }
}

// API-Versionsnummer zurückgeben
fun Route.version() {
    get("/") {
        call.respondText { "Eval API 1.0" }
    }
}

 // erwartet One-Time-Login-Daten und verifiziert diese;
 // wenn OK, ersetzt der appkey den onetimekey (wird in
 // Students-Tabelle gespeichert)
fun Route.login() {
     post("/login") {
         try {
             val logindata = call.receive<LoginData>()
             call.respond(db.oneTimeLogin(logindata))
         } catch (e: Exception) {
             println("Fehler in POST api/login: $e")
         }
     }
 }

// Verbindungstest mit Authentifizierung
fun Route.authtest() {
    get("/authtest") {
        call.respondText { "successfully authentitated" }
    }
}

// Liste aller für den Studenten relevanten Evaluationen ermitteln
// Achtung: {id} ist nicht die ID-Nummer des Studentens, sondern
// seine Matrikelnummer (studentNumber)
fun Route.evaluations() {
    get("/evaluations/{id}") {
        val studentId = call.parameters["id"] ?: "invalid id"
        call.respond(db.evaluations(studentId))
    }
}

// Details (Fragen) für eine bestimmte Evaluation ermitteln
fun Route.questions() {
    get("/questions/{id}") {
        val evalId = call.parameters["id"]?.toIntOrNull() ?: -1
        val q = db.questions(evalId)
        if (q is EvaluationDetails)
            call.respond(q)
        else
            call.respond("invalid data")
    }
}

// Evaluierung speichern; liefert false/true zurück
fun Route.evaluation() {
    post("/evaluation") {
        try {
            val evaldata = call.receive<EvaluationDetails>()
            println("xxx POST api/evaluation: data = $evaldata")
            call.respond(db.saveEvaluation(evaldata))
        } catch (e: Exception) {
            println("xxx Fehler in POST api/evaluation: $e")
            call.respond(false)
        }
    }
}

 // Evaluierungsergebnisse ermitteln
fun Route.results() {
     get("/results/{id}") {
         val evalId = call.parameters["id"]?.toIntOrNull() ?: -1
         val r = db.evaluationResult(evalId)
         // if (r is EvaluationDetails)
         call.respond(r)
         // else
         // call.respond("invalid data")
     }
 }