package info.kofler.eval

import kotlinx.serialization.Serializable

// Datenklassen zur Speicherung von Evaluierungsdaten
// sowie zum Datenaustausch über die Weboberfläche
// bzw. die API (dort JSON-Serialisierung)

// Einladung zu einer Evalierung
data class Invitation(val key: String,
                      val course: String,
                      val student: String)

// Metadaten einer Evalulierung (für Weboberfläche)
data class Evaluation(val id: Int,
                      val course: String,
                      val active: Boolean,
                      val participants: Int,
                      val invitations: Int)

// Metadaten einer Evalulierung (für REST-API)
@Serializable
data class EvaluationMeta(val evalid: Int,
                          val courseName: String,
                          val courseTime: String,
                          val evalEndDate: String,
                          val active: Boolean,
                          val open: Boolean)

// Details einer Evaluierung (inkl. Liste aller Fragen/Antworten)
@Serializable
data class EvaluationDetails(val evalId: Int,
                             val course: String = "",
                             val items: List<EvaluationItem>,
                             val studentId: Int? = null,
                             val studentNo: String? = null,
                             val invitations: Int = 0,
                             val participants: Int = 0)

// Details zu einer Frage: Fragetext, Fragetyp, Antwort)
@Serializable
data class EvaluationItem(val questId: Int,
                          val question: String = "",
                          val qtype: String = "",  // "range" oder "text"
                          val nmb: Int? = null,
                          val answer: String? = null)