package info.kofler.evalclient

import android.content.Context
import android.view.LayoutInflater
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import androidx.navigation.findNavController
import androidx.recyclerview.widget.RecyclerView
import info.kofler.evalclient.databinding.ItemEvalBinding

class Holder_Eval(binding: ItemEvalBinding) : RecyclerView.ViewHolder(binding.root) {
    val evalname: TextView   = binding.lblEvalname  // -> layout/res/item_eval.xml
    val evaltime: TextView   = binding.lblEvaltime
    val evalstatus: TextView = binding.lblEvalstatus
    val evalimg: ImageView   = binding.imgStatus
}

class Adapter_Eval(private val items: List<EvaluationMeta>,
                   private val context: Context)
    : RecyclerView.Adapter<Holder_Eval>()
{
    // Anzahl der Elemente
    override fun getItemCount() = items.size

    // leere Listenelemente erzeugen
    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): Holder_Eval {
        val binding = ItemEvalBinding.inflate(LayoutInflater.from(parent.context), parent, false)
        return Holder_Eval(binding)
    }

    // Listenelemente mit Daten befüllen
    override fun onBindViewHolder(holder: Holder_Eval, position: Int) {
        val item = items[position]
        val statusmsg = listOf(
            "Bitte evaluieren bis ${item.evalEndDate}",
            "Schon erledigt, aber Evaluation läuft noch",
            "Evaluierungsergebnisse liegen vor"
        )
        val colors = listOf(
            R.color.colorStatus0,
            R.color.colorStatus1,
            R.color.colorStatus2
        )
        val icons = listOf(
            R.drawable.eval_status_0,
            R.drawable.eval_status_1,
            R.drawable.eval_status_2
        )

        // je nach Status unterschiedliche Initialisierung
        holder.evalname.text = item.courseName
        holder.evaltime.text = item.courseTime
        val status = if (item.open) 0
                     else if (item.active) 1
                     else 2
        holder.evalstatus.text = statusmsg[status]
        holder.evalimg.setImageResource(icons[status])
        holder.evalimg.setColorFilter(context.getColor(colors[status]))

        // je nach Status unterschiedliche On-Click-Reaktion
        holder.itemView.setOnClickListener {
            shared.evalId = items[position].evalid
            if (status == 0)
                it.findNavController().navigate(R.id.action_fragmain_to_fragevalpost)
            else if(status == 2)
                it.findNavController().navigate(R.id.action_fragmain_to_fragevalresult)
        }
    }
}

