package info.kofler.evalclient
import android.content.Context
import android.text.Editable
import android.text.TextWatcher
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.EditText
import android.widget.RatingBar
import android.widget.TextView
import androidx.recyclerview.widget.RecyclerView
import info.kofler.evalclient.databinding.ItemPostevalBinding

class Holder_EvalPost(binding: ItemPostevalBinding) : RecyclerView.ViewHolder(binding.root) {
    val question: TextView = binding.lblQuestion  // -> layout/res/item_posteval.xml
    val stars: RatingBar   = binding.ratingBar
    val answer: EditText   = binding.editAnswer
}

class Adapter_EvalPost(private val items: List<EvaluationItem>,
                       private val context: Context)
    : RecyclerView.Adapter<Holder_EvalPost>()
{
    // Anzahl der Elemente
    override fun getItemCount() = items.size

    // leere Listenelemente erzeugen
    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): Holder_EvalPost {
        val binding = ItemPostevalBinding.inflate(LayoutInflater.from(parent.context), parent, false)
        return Holder_EvalPost(binding)
    }

    // Listenelemente mit Daten befüllen
    override fun onBindViewHolder(holder: Holder_EvalPost, position: Int) {
        val item = items[position]
        holder.question.text = item.question
        holder.answer.setText("")

        if (item.qtype == "range") {
            holder.stars.visibility = View.VISIBLE
            holder.answer.visibility = View.INVISIBLE
            // Bewertung speichern
            holder.stars.setOnRatingBarChangeListener { ratingBar: RatingBar, fl: Float, b: Boolean ->
                item.nmb = (fl / ratingBar.numStars * 100).toInt()
            }
        } else {
            holder.stars.visibility = View.INVISIBLE
            holder.answer.visibility = View.VISIBLE
            // Textänderungen speichern
            holder.answer.addTextChangedListener(object : TextWatcher {
                override fun afterTextChanged(s: Editable?) { }  // unnötig
                override fun beforeTextChanged(s: CharSequence?,
                    start: Int, count: Int, after: Int) { }      // auch unnötig
                override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {
                    item.answer = holder.answer.text.toString()
                }
            })
        }
    }
}

