package info.kofler.evalclient

import android.app.Activity
import android.os.Bundle
import android.util.Log
import androidx.fragment.app.Fragment
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.inputmethod.InputMethodManager
import androidx.navigation.fragment.findNavController
import androidx.recyclerview.widget.DividerItemDecoration
import androidx.recyclerview.widget.LinearLayoutManager
import info.kofler.evalclient.databinding.FragmentEvaluationPostBinding
import io.ktor.client.request.get
import io.ktor.client.request.post
import io.ktor.client.request.url
import io.ktor.http.ContentType
import io.ktor.http.contentType
import kotlinx.coroutines.*
import java.lang.Exception


class Frag_EvalPost : Fragment(), CoroutineScope by MainScope() {
    private var _binding: FragmentEvaluationPostBinding? = null
    // This property is only valid between onCreateView and
    // onDestroyView!
    private val binding get() = _binding!!

    // vorerst leeres EvaluationDetails-Objekt, wird später mit
    // richtigen Daten initialisiert
    var evalDetails = EvaluationDetails(-1, "", listOf())

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?): View?
    {
        _binding = FragmentEvaluationPostBinding.inflate(inflater, container, false)
        // siehe Kommentar in Frag_Main
        binding.rclEvalpost.adapter = Adapter_EvalPost(listOf(), requireContext())
        binding.rclEvalpost.layoutManager = LinearLayoutManager(context)
        val divider = DividerItemDecoration(context, DividerItemDecoration.VERTICAL)
        binding.rclEvalpost.addItemDecoration(divider)

        binding.btnEvalpostOk.setOnClickListener {
            Log.i("xxx", "EvalDetails = $evalDetails")
            saveEvaluation()
        }
        getEvalDetails()
        return binding.root
    }

    override fun onDestroy() {
        super.onDestroy()
        cancel()  // Hintergrundprozesse stoppen
    }

    // Evaluierungsdetails (also Fragestellung) ermitteln
    private fun getEvalDetails() {
        launch(Dispatchers.IO) {
            try {
                val evalId = shared.evalId
                // Evaluierungsfragen vom Server herunterladen
                evalDetails = authClient.get("$evalhost/api/questions/$evalId")
                evalDetails.studentNo = shared.studentNo

                launch(Dispatchers.Main) {
                    binding.lblEvalpostname.text = evalDetails.course
                    // neue Instanz von evalDetails, daher neuer Adapter
                    binding.rclEvalpost.adapter = Adapter_EvalPost(evalDetails.items, requireContext())
                }
            } catch(e: Exception) {
                Log.e("xxx", "EvaluationPostFragment, getEvalDetails: $e")
            }
        }
    }

    // Evaluierung speichern
    private fun saveEvaluation() {
        launch(Dispatchers.IO) {
            try {
                // Evaluierungsergebnis speichern (POST)
                val ok = authClient.post<Boolean> {
                    url("$evalhost/api/evaluation")
                    contentType(ContentType.Application.Json)
                    body = evalDetails
                }

                launch(Dispatchers.Main) {
                    if (ok) {
                        // Tastatur ausblenden
                        val imm = context?.getSystemService(
                            Activity.INPUT_METHOD_SERVICE) as InputMethodManager
                        imm.hideSoftInputFromWindow(view?.windowToken, 0)
                        // zurück ins Main Fragment
                        findNavController().navigateUp()
                    }
                    else
                        // Fehlermeldung anzeigen
                        binding.lblError.visibility = View.VISIBLE
                }
            }  catch(e: Exception) {
                Log.e("xxx", "EvaluationPostFragment, saveEvaluation: $e")
                launch(Dispatchers.Main) {
                    // Fehlermeldung anzeigen
                    binding.lblError.visibility = View.VISIBLE
                }
            }
        }
    }
}

