package info.kofler.evalclient

import android.app.Activity
import android.content.Context
import android.os.Bundle
import android.util.Log
import androidx.fragment.app.Fragment
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.inputmethod.InputMethodManager
import android.widget.TextView
import androidx.navigation.NavController
import androidx.navigation.fragment.NavHostFragment.findNavController
import info.kofler.evalclient.databinding.FragmentLoginBinding
import io.ktor.client.request.post
import io.ktor.client.request.url
import io.ktor.http.ContentType
import io.ktor.http.contentType
import kotlinx.coroutines.*
import kotlinx.serialization.*

class Frag_Login : Fragment(), CoroutineScope by MainScope() {
    private var _binding: FragmentLoginBinding? = null
    // This property is only valid between onCreateView and
    // onDestroyView!
    private val binding get() = _binding!!

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?): View?
    {
        // neues Fragment zwischenspeichern
        _binding = FragmentLoginBinding.inflate(inflater, container, false)
        binding.btnOK.setOnClickListener {
            oneTimeLogin(binding.editStudentNr.text.toString(),
                         binding.editKey.text.toString(),
                         findNavController(this),
                         binding.lblError)
        }
        // Fragment zurückgeben
        return binding.root
    }
    override fun onDestroy() {
        super.onDestroy()
        cancel()  // Hintergrundprozesse stoppen
    }


    // Login mit POST-Request versuchen; wenn OK, wird zufälliges
    // Passwort gespeichert, das in Zukunft für die reguläre Authentifizierung
    // verwendet wird
    private fun oneTimeLogin(studNo: String,
                             oneTimeKey: String,
                             navCtrl: NavController,
                             lblErrorMsg: TextView)  {
        // zufälliger App-Key, wird lokal gespeichert, wenn Login erfolgreich
        val chars = "0123456789abcdefghijklmnopqstuvwxyz"
        val appkey = (1..20).map { chars.random() }.joinToString("")

        // login-Triplet zusammenstellen, Leerzeichen + Whitespace entfernen
        val stud = studNo.replace(" ", "")
        val one = oneTimeKey.replace(" ", "")
        val login = LoginData(stud, one, appkey)

        // POST-Request
        launch(Dispatchers.IO) {
            var ok = false
            try {
                ok = httpClient.post<Boolean> {
                    url("$evalhost/api/login")
                    contentType(ContentType.Application.Json)
                    body = login
                }
            } catch(e: Exception) {
                Log.e("xxx", "Fehler bei /api/login: $e")
            }
            launch(Dispatchers.Main) {
                if (ok) {
                    // in SharedData speichern, damit in allen Fragmenten zugänglich
                    shared.studentNo = stud
                    shared.appkey = appkey

                    // dauerhaft speichern, damit auch beim nächsten App-Start verfügbar
                    val pref = requireActivity().getPreferences(Context.MODE_PRIVATE)
                    pref.edit().putString("studentId", stud).apply()
                    pref.edit().putString("appkey", appkey).apply()

                    // authClient neu einrichten, damit die
                    // Authentifizierungsdaten verwendet werden
                    authClient = setupAuthClient(stud, appkey)

                    // Tastatur ausblenden
                    val imm = context?.getSystemService(
                        Activity.INPUT_METHOD_SERVICE) as InputMethodManager
                    imm.hideSoftInputFromWindow(view?.windowToken, 0)

                    // zur Hauptseite
                    navCtrl.navigateUp()
                } else {
                    // Fehlermeldung anzeigen
                    lblErrorMsg.visibility = View.VISIBLE
                }
            }
        }
    }


    @Serializable
    private data class LoginData (val studentNo: String,
                                  val onetimekey: String,
                                  val appkey: String)
}







