package info.kofler.evalclient

import android.os.Bundle
import android.util.Log
import androidx.fragment.app.Fragment
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.navigation.NavController
import androidx.navigation.fragment.findNavController
import androidx.recyclerview.widget.DividerItemDecoration
import androidx.recyclerview.widget.LinearLayoutManager
import info.kofler.evalclient.databinding.FragmentMainBinding
import io.ktor.client.request.get
import kotlinx.coroutines.*


class Frag_Main : Fragment(), CoroutineScope by MainScope() {
    private var _binding: FragmentMainBinding? = null
    // This property is only valid between onCreateView and
    // onDestroyView!
    private val binding get() = _binding!!

    var eval = listOf<EvaluationMeta>()        // hier wird später die Liste aller Evaluierungen gespeichert
    // var adapter: Adapter_Eval? = null

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?): View
    {
        // Fragment erzeugen
        _binding = FragmentMainBinding.inflate(inflater, container, false)
        // Recycler vorerst mit leerem Adapter initialisieren
        // verhindert Fehlermeldung: 'RecyclerView: No adapter
        // attached; skipping layout', weil der Adapter erst später
        // (asynchron) eingerichtet wird
        binding.rclEval.adapter = Adapter_Eval(eval, requireContext())
        binding.rclEval.layoutManager = LinearLayoutManager(context)
        // Trennlinie zwischen den Listeneinträgen
        val divider = DividerItemDecoration(context, DividerItemDecoration.VERTICAL)
        binding.rclEval.addItemDecoration(divider)
        return binding.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        // Evaluierungs-Liste initialisieren oder bei
        // verbindungsproblemen in ein anderes Fragment
        // wechseln
        testConnection(findNavController())
    }

    override fun onDestroy() {
        super.onDestroy()
        cancel()
    }

    // wird vom Main-Fragment aufgerufen
    // testet, ob Verbindung zum Evaluierungs-Server möglich ist;
    // falls nicht, Wechsel in das entsprechende Fragment:
    //   * NoConnection-Fragment wenn kein Verbindungsaufbau möglich
    //   * Login-Fragment, wenn zwar Verbindungsaufbau möglich,
    //     aber Authentifizierung nicht möglich
    // wenn Authentifizierung möglich: Liste der Evaluierungen herunterladen
    private fun testConnection(navCtrl: NavController) {
        launch(Dispatchers.IO) {
            try {
                // 1: testet, ob Kontakt zum Server möglich ist
                val msg1 = httpClient.get<String>("$evalhost/api/")

                // 2: Sind studentNo und appkey schon bekannt?
                if (shared.appkey.isBlank() || shared.studentNo.isBlank())
                {
                    // nein -> Wechseln in das LoginFragment
                    launch(Dispatchers.Main) {
                        navCtrl.navigate(
                            R.id.action_fragmain_to_fraglogin)
                    }
                    // Abbruch des Codes in launch(Dispatchers.IO)
                    return@launch
                }

                // 2a: Login mit Authentifizierung versuchen
                //     nur zum Debugging
                // authClient.get<String>("$evalhost/api/authtest")

                // 3: Liste der Evaluierungen herunterladen
                initAdapter()
            } catch(e: Exception) {
                launch(Dispatchers.Main) {
                    Log.e("xxx",  "Verbindungsprobleme: $e")
                    // Wechseln in das No-Login-Fragment
                    navCtrl.navigate(
                        R.id.action_fragmain_to_fragnoconnect)
                }
            }
        }
    }

    private suspend fun initAdapter() {
        try {
            val studNo = shared.studentNo
            eval = authClient.get("$evalhost/api/evaluations/$studNo")
            launch(Dispatchers.Main) {
                // neuer Adapter, weil neue Instanz von eval
                binding.rclEval.adapter = Adapter_Eval(eval, requireContext())
            }
        } catch(e: Exception) {
            Log.e("xxx", "Reading evaluations failed $e")
        }
    }

}
