package info.kofler.evalclient

import android.os.Build

import io.ktor.client.HttpClient
import io.ktor.client.features.auth.Auth
import io.ktor.client.features.auth.providers.basic
import io.ktor.client.features.json.JsonFeature
import io.ktor.client.features.json.serializer.KotlinxSerializer


// globale Variablen
val evalhost =
    // 10.0.2.2 = Hostrechner aus Sicht des Emulators
    if (isEmulator()) "http://10.0.2.2:8080"
    // 192.168.178.54: IP-Adresse des Entwicklungsrechner, anpassen!
    else              "http://192.168.178.54:8080"

val shared = SharedData()  // -> Definition in Model.kt

// Ktor-HttpClient 1: ohne Authentifizierung
// (für Verbindungstest und Einmal-Login)
val httpClient = HttpClient() {
    install(JsonFeature) {
        serializer = KotlinxSerializer()
    }
}

// Client 2: mit Authentifizierung
// (für alle weiteren Aufgaben)
// var, weil der Client im Programmverlauf
// mehrfach neu eingerichtet wird, wenn
// die Authentifizierungsdaten bekannt sind
// var authClient = HttpClient()

lateinit var authClient : HttpClient

fun setupAuthClient(name: String, pw: String)  =
    HttpClient() {
        install(JsonFeature) {
            serializer = KotlinxSerializer()
        }
        install(Auth) {
            basic {
                sendWithoutRequest = true  // https://stackoverflow.com/questions/61362485
                username = name
                password = pw
            }
        }
    }


// testet, ob App im Emulator läuft
// besserer Test: siehe https://stackoverflow.com/questions/2799097
fun isEmulator() =
    Build.FINGERPRINT.contains("generic_x86")



