package info.kofler.evalclient

import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import kotlinx.serialization.Serializable

// Datenmodell (Klassen, die an mehreren Stellen
// im Code benötigt werden)

// App-Daten, die zwischen der Activity und den
// Fragmenten geteilt werden
class SharedData {
    var isInitialized = false

    // zur Authentifizierung beim Eval-Server:
    // Studenten/Schüler-Nummer ('Matrikelnummer')
    // und ein app-spezifischer Key
    var studentNo = ""
    var appkey = ""

    // die zuletzt ausgewählte Evaluierung
    var evalId = -1
}

// die folgenden drei Klassen sind synchron mit den
// gleichnamigen Klassen im Projekt eval-backend,
// Datei model.kt

// Metadaten einer Evaluierung (Name, Aktivität)
@Serializable
data class EvaluationMeta(val evalid: Int,
                          val courseName: String,
                          val courseTime: String,
                          val evalEndDate: String,
                          val active: Boolean,
                          val open: Boolean)

// Details einer Evaluierung inkl. Fragen + Antworten)
@Serializable
data class EvaluationDetails(val evalId: Int,
                             val course: String = "",
                             val items: List<EvaluationItem>,
                             val studentId: Int? = null,
                             var studentNo: String? = null,
                             val invitations: Int = 0,
                             val participants: Int = 0)

// Details zu einer Frage einer Evaluierung
// (Fragetext, Fragetyp, Antwort)
@Serializable
data class EvaluationItem(val questId: Int,
                          val question: String = "",
                          val qtype: String = "",
                          var nmb: Int? = null,
                          var answer: String? = null)